/****************The Lytic Group, copyright, 2021*****************/

/****************Published March, 2021 *****************/

/*********from http://www.lyticgroup.com by Edward Heraux*****************/





USE [AdventureWorks]

GO

/****** Object:  StoredProcedure [dbo].[uspCheckDBSize]    Script Date: 05/11/2020 13:42:35 ******/

SET ANSI_NULLS ON

GO

SET QUOTED_IDENTIFIER ON

GO

CREATE PROC [dbo].[uspLogDBSize]

 @AllDbs bit=1 --Whether to capture this data for all databases on this server at once (1)

				--or only the current datbase's info (0)

			



/************************************************************************************************

Schedule this to run weekly or monthly, and give yourself quick access to patterns in your databases' growth.

Store this proc in the database you're most likely to check up on the most, as that will be the default db

for which this proc captures size data, if you pass a parameter of 0.



This stored procedure stores snapshots of database size information at the time you run it,

either for all your databases or just the one from which you execute it.

Records go into a permanent table (dbo.tblSizeHistory), off of which you can run reports or just query

for size histories or trends.

Bundled below, however, are UDFs and a view that feed an SSRS report.



Also included with your download are SSRS reports named DBSizeDashboard2005.rdl and DBSizeDashboard2008.rdl,

which give you a graphical look at all the results from that user-defined function.



 *******************************************************************************************/

AS

SET NOCOUNT ON



 begin try 

	BEGIN TRAN

		IF NOT EXISTS (SELECT 1 FROM sys.sysobjects WHERE xtype='U' AND Name='tblDBSizeHistory')

			CREATE TABLE dbo.tblDBSizeHistory

				(DBName sysname

				,FileID int

				,FileSizeInMB money

				,SpaceUsedInMB money

				,FreeSpaceInMB money

				,LogicalFileName sysname

				,FilePath varchar(500)

				,AsOfDateTime datetime

				CONSTRAINT PK_PathAndTime PRIMARY KEY CLUSTERED(DBName,FilePath,AsOfDateTime))

		

			

		--Just for testing

		IF EXISTS (SELECT 1 FROM Tempdb.sys.sysobjects WHERE Name LIKE '#DBFileStats%')

			DROP TABLE #DBFileStats

		IF EXISTS (SELECT 1 FROM TempDB.sys.sysobjects WHERE Name LIKE '#DBList%')

			DROP TABLE #DBList 

			

		--Create a temp table of all databases on this server, which we will be looping through		

		SELECT dbid,Name INTO #DBList FROM master.sys.sysdatabases

	COMMIT	 

	--Variables for the loop

	DECLARE @counter int, @NumDatabases int

	DECLARE @DBName sysname

	DECLARE @SQL varchar(750), @CurrentDB sysname

	 

	--Set up values for the loop

	IF @AllDbs=1

		SELECT @NumDatabases=COUNT(*), @counter=1, @CurrentDB='%' FROM #DBlist

	ELSE

		SELECT @NumDatabases=1, @counter=1, @CurrentDB=DB_NAME() FROM #DBlist

	

	--Loop through the list of databases,

	-- inserting each one's filesize info into the tblDBSizeHistory permanent table	

	BEGIN TRAN		 

	  WHILE @Counter <= @NumDatabases

		BEGIN

			--There will only be one record in #DBList to loop through

			-- if an @AllDBs parameter of 1 was passed 

			SELECT TOP 1 @DBName=Name FROM #DBList WHERE Name LIKE @CurrentDB 

			--Insert a row for each file of the current database

			SET @SQL = 'USE ' + @DBName  

				+ ' INSERT INTO ' +DB_NAME() + '.dbo.tblDBSizeHistory 

				SELECT ''' + @DBName  + ''' AS DBName, FileID

			  ,convert(decimal(12,2),round(size/128.000,2)) AS FileSizeInMB

				,convert(decimal(12,2),round(fileproperty(name,''SpaceUsed'')/128.000,2))AS SpaceUsedInMB

				,convert(decimal(12,2),round((size-fileproperty(name,''SpaceUsed''))/128.000,2)) AS FreeSpaceInMb

				,NAME AS LogicalFileName, Filename AS FilePath

				,Getdate() AS AsOfDateTime

				FROM ' + @DBName + '.sys.sysfiles'

			EXECUTE(@SQL)

			DELETE FROM #DBList WHERE Name=@DBName

			SET @counter=@counter+1

		END   

		

		--Just for testing, show the results (i.e. the most recent file sizes)

		--SELECT CONVERT(varchar(20),AsOfDateTime,100) AS RunTimeToMinute

		--,CONVERT(varchar(20),(SELECT MAX(AsOfDateTime) FROM dbo.tblDBSizeHistory),100) AS MaxTime

		--,* 

		--FROM dbo.tblDBSizeHistory 

		--WHERE CONVERT(varchar(20),AsOfDateTime,100)=CONVERT(varchar(20),(SELECT MAX(AsOfDateTime) FROM dbo.tblDBSizeHistory),100)

	COMMIT

end try 



begin catch 

	--We're going to refer to these variables in a RAISERROR command

	DECLARE @ErrorMessage NVARCHAR(4000)

	DECLARE @ErrorSeverity INT

	DECLARE @ErrorState INT



	SELECT @ErrorMessage = 'Error number ' + CAST(ERROR_NUMBER() AS varchar(10)) +': ' + ERROR_MESSAGE(),

		  @ErrorSeverity = ERROR_SEVERITY(),

		  @ErrorState = ERROR_STATE()

           

	RAISERROR(@ErrorMessage,@ErrorSeverity,@ErrorState) 

	ROLLBACK TRAN

end catch 



GO



/*********************************************************************************

This table-valued function will feed one of the datasets

 for the reports included in your download.

*********************************************************************************/



CREATE FUNCTION dbo.ufnDBSizeHistory 

(	@StartDate datetime, @EndDate datetime

)

/****************The Lytic Group, copyright, 2021*****************/

/****************Published March, 2021 *****************/

/*********from http://www.lyticgroup.com by Edward Heraux*****************/



/*Find all the size records time-stamped between the two date parameters*/

RETURNS TABLE 

AS

RETURN 

	SELECT *, SpaceUsedInMB/FileSizeInMB AS PercentFull

		,CASE RIGHT(FilePath,3) WHEN 'ldf' THEN 'Log File' ELSE 'Data File' END AS FileType 

	FROM dbo.tblDBSizeHistory

	WHERE CONVERT(char(12),AsOfDateTime) BETWEEN CONVERT(char(12),@StartDate) AND CONVERT(char(12),@EndDate)

GO







/****************The Lytic Group, copyright, 2021*****************/

/****************Published March, 2021 *****************/

/***





/********************************************************************************

This table-valued function returns one row with info about the current server.

This is included strictly to feed a header in the included reports.

**********************************************************************************/



IF OBJECT_ID (N'dbo.ufnServerInfo', N'IF') IS NOT NULL

    DROP FUNCTION dbo.ufnServerInfo

GO



CREATE FUNCTION dbo.ufnServerInfo()

******from http://www.lyticgroup.com by Edward Heraux*****************/



RETURNS TABLE

AS

RETURN

--Grab server name and version info

SELECT SERVERPROPERTY('servername') AS ServerName

	, @@Version as SQLServerVersion

	, CONVERT(VARCHAR(100), serverproperty('Edition')) AS Edition

	, SERVERPROPERTY('productversion')AS Version



	, SERVERPROPERTY ('productlevel') AS ProductLevel

	,CONVERT(VARCHAR(100), SERVERPROPERTY ('LicenseType')) AS LicenseType

	

GO



/*************************************************************************************/

/*For a couple of the graphs included in the report, 

find the most recent file sizes */

**************************************************************************************/



CREATE VIEW [dbo].vLatestDBSizes 

AS

/****************The Lytic Group, copyright, 2021*****************/

/****************Published March, 2021 *****************/

/*********from http://www.lyticgroup.com by Edward Heraux*****************/





SELECT M.DBName, M.FilePath, M.LatestTimestamp 

	,CASE RIGHT(H.FilePath,3) WHEN 'ldf' THEN 'Log File' ELSE 'Data File' END AS FileType

	,H.FileSizeInMB, H.SpaceUsedInMB,H.LogicalFileName, H.SpaceUsedInMB/H.FileSizeInMB AS PercentFull

FROM tblDBSizeHistory AS H

	INNER JOIN

		(SELECT DBName, FilePath

		,MAX(AsOfDateTime) AS LatestTimestamp 

		FROM tblDBSizeHistory 

		GROUP BY  DBName,FilePath) AS M

	ON M.DBName=H.DBName AND H.AsOfDateTime=M.LatestTimestamp AND H.FilePath=M.FilePath

